#ifndef CALENDAR_H
#define CALENDAR_H

#include <glib.h>

#include <mimedir/mimedir-valarm.h>
#include <mimedir/mimedir-vcomponent.h>
#include <mimedir/mimedir-vevent.h>

#include "gnomecal-alarm-sink.h"
#include "todo-list.h"

G_BEGIN_DECLS

typedef int (*calendarfn) (MIMEDirVComponent *, time_t, time_t, void *);

typedef enum {
        CHANGE_NEW     = 1 << 0,        /* new object */
        CHANGE_SUMMARY = 1 << 1,        /* summary */
        CHANGE_DATES   = 1 << 2,        /* dtstart / dtend */
        CHANGE_ALL     = CHANGE_SUMMARY | CHANGE_DATES
} CalObjectChange;

typedef struct {
	GncalAlarmSink *alarm_sink;

	/* The list of events and journal entries */
	GList  	*events;
	GList  	*journal;

	GHashTable *event_hash;
	
	/* Time at which the calendar was created */
	time_t  created;

	/* If the calendar was last modified */
	int     modified;
	void    *temp;

	void    *corba_server;
} Calendar;

/* This is only used by the calendar_get_events_in_range routine to get
 * a list of objects that recur on a specific date
 */
typedef struct {
	time_t     ev_start;
	time_t     ev_end;
	MIMEDirVEvent *event;
} CalendarObject;

typedef enum {
	CALENDAR_INIT_NIL    = 0,
	CALENDAR_INIT_ALARMS = 1 << 0
} CalendarNewOptions;

Calendar *calendar_new                  (void);
void      calendar_free                 (Calendar *cal);
void      calendar_add_object           (Calendar *cal, MIMEDirVComponent *obj);
void      calendar_remove_object        (Calendar *cal, MIMEDirVComponent *obj);
void      calendar_destroy              (Calendar *cal);

void      calendar_iterate              (Calendar *cal, time_t start, time_t end, calendarfn cb, void *closure);

/* Returns a list of CalendarObject structures.  These represent the events in the calendar that are
 * in the specified range.
 */
GList *calendar_get_events_in_range (Calendar *cal, time_t start, time_t end);

/* Destroy list returned by calendar_get_events_in_range() with this function */
void calendar_destroy_event_list (GList *l);

void      calendar_notify (time_t time, MIMEDirVAlarm *which, void *data);

GObject  *calendar_object_find_event    (Calendar *cal, const char *uid);
GObject  *calendar_object_find_todo     (Calendar *cal, const char *uid);
GObject  *calendar_object_find          (Calendar *cal, const char *uid);

void      calendar_set_day (void);

G_END_DECLS

#endif
